@@ -3,11 +3,11 @@ use strict;
 use warnings;
 use parent "DBIx::DataModel::Meta";
 use DBIx::DataModel;
-use DBIx::DataModel::Meta::Utils;
+use DBIx::DataModel::Meta::Utils qw/define_readonly_accessors/;
 
-use Scalar::Util         qw/looks_like_number weaken/;
-use Params::Validate     qw/validate SCALAR HASHREF ARRAYREF OBJECT/;
-use Carp;
+use Scalar::Util                 qw/looks_like_number weaken/;
+use Params::Validate             qw/validate_with SCALAR HASHREF ARRAYREF OBJECT/;
+use Carp::Clan                   qw[^(DBIx::DataModel::|SQL::Abstract)];
 use namespace::clean;
 
 {no strict 'refs'; *CARP_NOT = \@DBIx::DataModel::CARP_NOT;}
@@ -27,7 +27,11 @@ sub new {
   my $class = shift;
 
   # parse arguments and create $self
-  my $self = validate(@_, $path_spec);
+  my $self = validate_with(
+    params      => \@_,
+    spec        => $path_spec,
+    allow_extra => 0,
+   );
 
   my $path = $self->{name};
   weaken $self->{$_} for qw/from to association/;
@@ -49,9 +53,7 @@ sub new {
   bless $self, $class;
 }
 
-DBIx::DataModel::Meta::Utils->define_readonly_accessors(
-  __PACKAGE__, keys %$path_spec
-);
+define_readonly_accessors(__PACKAGE__, keys %$path_spec);
 
 
 sub opposite {
