@@ -3,12 +3,11 @@ use strict;
 use warnings;
 use parent "DBIx::DataModel::Meta";
 use DBIx::DataModel;
+use DBIx::DataModel::Meta::Utils qw/define_readonly_accessors does/;
 
 use Scalar::Util         qw/weaken/;
-use Scalar::Does         qw/does/;
-use Params::Validate     qw/validate OBJECT SCALAR HASHREF/;
-use Carp;
-
+use Params::Validate     qw/validate_with OBJECT SCALAR HASHREF/;
+use Carp::Clan           qw[^(DBIx::DataModel::|SQL::Abstract)];
 use namespace::clean;
 
 {no strict 'refs'; *CARP_NOT = \@DBIx::DataModel::CARP_NOT;}
@@ -17,11 +16,15 @@ sub new {
   my $class = shift;
 
   # parse arguments and create $self
-  my $self = validate(@_, {
-    schema   => {type => OBJECT, isa  => "DBIx::DataModel::Meta::Schema"},
-    name     => {type => SCALAR},
-    handlers => {type => HASHREF},
-  });
+  my $self = validate_with(
+    params      => \@_,
+    spec        => {
+      schema   => {type => OBJECT, isa  => "DBIx::DataModel::Meta::Schema"},
+      name     => {type => SCALAR},
+      handlers => {type => HASHREF},
+    },
+    allow_extra => 0,
+   );
 
   while (my ($name, $body) = each %{$self->{handlers}}) {
     does($body, 'CODE')
@@ -36,10 +39,7 @@ sub new {
 
 
 # accessor methods
-DBIx::DataModel::Meta::Utils->define_readonly_accessors(
-  __PACKAGE__, qw/schema name handlers/
-);
-
+define_readonly_accessors(__PACKAGE__, qw/schema name handlers/);
 
 
 1;
